@@ -76,7 +76,7 @@ static void databases_items_xxx (DMAPShare *share,
 				 GHashTable *query,
 				 SoupClientContext *context);
 static struct DMAPMetaDataMap *get_meta_data_map (DMAPShare *share);
-static void add_entry_to_mlcl (gpointer id,
+static guint32 add_entry_to_mlcl (gpointer id,
 			       DMAPRecord *record,
 			       gpointer mb);
 
@@ -287,7 +287,7 @@ daap_share_server_info (DMAPShare *share,
 	dmap_structure_add (msrv, DMAP_CC_MSLR, 0);
 	dmap_structure_add (msrv, DMAP_CC_MSTM, (gint32) DAAP_TIMEOUT);
 	dmap_structure_add (msrv, DMAP_CC_MSAL, (gchar) 0);
-	dmap_structure_add (msrv, DMAP_CC_MSUP, (gchar) 0);
+	dmap_structure_add (msrv, DMAP_CC_MSUP, (gchar) 1);
 	dmap_structure_add (msrv, DMAP_CC_MSPI, (gchar) 0);
 	dmap_structure_add (msrv, DMAP_CC_MSEX, (gchar) 0);
 	dmap_structure_add (msrv, DMAP_CC_MSBR, (gchar) 0);
@@ -339,7 +339,14 @@ typedef enum {
 	SONG_TRACK_NUMBER,
 	SONG_USER_RATING,
 	SONG_YEAR,
-	SONG_HAS_VIDEO
+	SONG_HAS_VIDEO,
+	SONG_SMART_PLAYLIST,
+	SONG_IS_PODCAST_PLAYLIST,
+	SONG_SPECIAL_PLAYLIST,
+	SONG_SAVED_GENIUS,
+	SONG_MEDIAKIND,
+	HAS_CHILD_CONTAINERS,
+	PARENT_CONTAINER_ID
 } DAAPMetaData;
 
 static struct DMAPMetaDataMap meta_data_map[] = {
@@ -380,6 +387,13 @@ static struct DMAPMetaDataMap meta_data_map[] = {
 	{"daap.sortalbum",		SONG_SORT_ALBUM},
 	{"daap.sortartist",		SONG_SORT_ARTIST},
 	{"com.apple.itunes.has-video",	SONG_HAS_VIDEO},
+	{"com.apple.itunes.smart-playlist",		SONG_SMART_PLAYLIST},
+	{"com.apple.itunes.is-podcast-playlist",	SONG_IS_PODCAST_PLAYLIST},
+	{"com.apple.itunes.special-playlist",		SONG_SPECIAL_PLAYLIST},
+	{"com.apple.itunes.saved-genius",		SONG_SAVED_GENIUS},
+	{"com.apple.itunes.mediakind",			SONG_MEDIAKIND},
+	{"dmap.haschildcontainers",			HAS_CHILD_CONTAINERS},
+	{"dmap.parentcontainerid",			PARENT_CONTAINER_ID},
 	{ NULL,				0}};
 
 #define DAAP_ITEM_KIND_AUDIO 2
@@ -528,14 +542,13 @@ send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record,
 	/* NOTE: cd g_free'd by chunked_message_finished(). */
 }
 
-static void
+static guint32
 add_entry_to_mlcl (gpointer id,
 		   DMAPRecord *record,
 		   gpointer _mb)
 {
 	GNode *mlit;
-	struct MLCL_Bits *mb;
-	mb = (struct MLCL_Bits *) _mb;
+	struct MLCL_Bits *mb = (struct MLCL_Bits *) _mb;
 	mlit = dmap_structure_add (mb->mlcl, DMAP_CC_MLIT);
 
 	if (_dmap_share_client_requested (mb->bits, ITEM_KIND))
@@ -543,10 +556,13 @@ add_entry_to_mlcl (gpointer id,
 	if (_dmap_share_client_requested (mb->bits, ITEM_ID))
 		dmap_structure_add (mlit, DMAP_CC_MIID, GPOINTER_TO_UINT (id));
 	if (_dmap_share_client_requested (mb->bits, ITEM_NAME)) {
-		gchar *title;
+		gchar *title = NULL;
 		g_object_get (record, "title", &title, NULL);
-		dmap_structure_add (mlit, DMAP_CC_MINM, title);
-		g_free (title);
+		if (title) {
+			dmap_structure_add (mlit, DMAP_CC_MINM, title);
+			g_free (title);
+		} else
+			g_warning ("Title requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, PERSISTENT_ID))
 		dmap_structure_add (mlit, DMAP_CC_MPER, GPOINTER_TO_UINT (id));
@@ -559,21 +575,27 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASUL, "daap://192.168.0.100:%u/databases/1/items/%d.%s?session-id=%s", data->port, *id, daap_record_get_format (DAAP_RECORD (record)), data->session_id);
 	*/
 	if (_dmap_share_client_requested (mb->bits, SONG_ALBUM)) {
-		gchar *album;
-		g_object_get (record, "daap.songalbum", &album, NULL);
-		dmap_structure_add (mlit, DMAP_CC_ASAL, album);
-		g_free (album);
+		gchar *album = NULL;
+		g_object_get (record, "songalbum", &album, NULL);
+		if (album) {
+			dmap_structure_add (mlit, DMAP_CC_ASAL, album);
+			g_free (album);
+		} else
+			g_warning ("Album requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_GROUPING))
 		dmap_structure_add (mlit, DMAP_CC_AGRP, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_ARTIST)) {
-		gchar *artist;
-		g_object_get (record, "daap.songartist", &artist, NULL);
-		dmap_structure_add (mlit, DMAP_CC_ASAR, artist);
-		g_free (artist);
+		gchar *artist = NULL;
+		g_object_get (record, "songartist", &artist, NULL);
+		if (artist) {
+			dmap_structure_add (mlit, DMAP_CC_ASAR, artist);
+			g_free (artist);
+		} else
+			g_warning ("Artist requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_BITRATE)) {
-		gint32 bitrate;
+		gint32 bitrate = 0;
 		g_object_get (record, "bitrate", &bitrate, NULL);
 		if (bitrate != 0)
 			dmap_structure_add (mlit, DMAP_CC_ASBR, (gint32) bitrate);
@@ -587,19 +609,19 @@ add_entry_to_mlcl (gpointer id,
 	if (_dmap_share_client_requested (mb->bits, SONG_COMPOSER))
 		dmap_structure_add (mlit, DMAP_CC_ASCP, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_DATE_ADDED)) {
-		gint32 firstseen;
+		gint32 firstseen = 0;
 		g_object_get (record, "firstseen", &firstseen, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDA, firstseen);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_DATE_MODIFIED)) {
-		gint32 mtime;
+		gint32 mtime = 0;
 		g_object_get (record, "mtime", &mtime, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDM, mtime);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_DISC_COUNT))
 		dmap_structure_add (mlit, DMAP_CC_ASDC, (gint32) 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_DISC_NUMBER)) {
-		gint32 disc;
+		gint32 disc = 0;
 		g_object_get (record, "disc", &disc, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDN, disc);
 	}
@@ -608,19 +630,25 @@ add_entry_to_mlcl (gpointer id,
 	if (_dmap_share_client_requested (mb->bits, SONG_EQ_PRESET))
 		dmap_structure_add (mlit, DMAP_CC_ASEQ, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_FORMAT)) {
-		gchar *format;
+		gchar *format = NULL;
 		if (transcode_format)
 			format = g_strdup (transcode_format);
 		else
 			g_object_get (record, "format", &format, NULL);
-		dmap_structure_add (mlit, DMAP_CC_ASFM, format);
-		g_free (format);
+		if (format) {
+			dmap_structure_add (mlit, DMAP_CC_ASFM, format);
+			g_free (format);
+		} else
+			g_warning ("Format requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_GENRE)) {
-		gchar *genre;
-		g_object_get (record, "daap.songgenre", &genre, NULL);
-		dmap_structure_add (mlit, DMAP_CC_ASGN, genre);
-		g_free (genre);
+		gchar *genre = NULL;
+		g_object_get (record, "songgenre", &genre, NULL);
+		if (genre) {
+			dmap_structure_add (mlit, DMAP_CC_ASGN, genre);
+			g_free (genre);
+		} else
+			g_warning ("Genre requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_DESCRIPTION))
 		dmap_structure_add (mlit, DMAP_CC_ASDT, ""); /* FIXME: e.g., wav audio file */
@@ -629,7 +657,7 @@ add_entry_to_mlcl (gpointer id,
 	if (_dmap_share_client_requested (mb->bits, SONG_SAMPLE_RATE))
 		dmap_structure_add (mlit, DMAP_CC_ASSR, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_SIZE)) {
-		guint64 filesize;
+		guint64 filesize = 0;
 		g_object_get (record, "filesize", &filesize, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASSZ, (gint32) filesize);
 	}
@@ -645,27 +673,27 @@ add_entry_to_mlcl (gpointer id,
 	if (_dmap_share_client_requested (mb->bits, SONG_TRACK_COUNT))
 		dmap_structure_add (mlit, DMAP_CC_ASTC, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_TRACK_NUMBER)) {
-		gint32 track;
+		gint32 track = 0;
 		g_object_get (record, "track", &track, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASTN, track);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_USER_RATING)) {
-		gint32 rating;
+		gint32 rating = 0;
 		g_object_get (record, "rating", &rating, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASUR, rating);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_YEAR)) {
-		gint32 year;
+		gint32 year = 0;
 		g_object_get (record, "year", &year, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASYR, year);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_HAS_VIDEO)) {
-		gboolean has_video;
+		gboolean has_video = 0;
 		g_object_get (record, "has-video", &has_video, NULL);
 		dmap_structure_add (mlit, DMAP_CC_AEHV, has_video);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_SORT_ARTIST)) {
-		gchar *sort_artist;
+		gchar *sort_artist = NULL;
 		g_object_get (record, "sort-artist", &sort_artist, NULL);
 		if (sort_artist)
 			dmap_structure_add (mlit, DMAP_CC_ASSA, sort_artist);
@@ -673,22 +701,29 @@ add_entry_to_mlcl (gpointer id,
 			g_warning ("Sort artist requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_SORT_ALBUM)) {
-		gchar *sort_album;
+		gchar *sort_album = NULL;
 		g_object_get (record, "sort-album", &sort_album, NULL);
 		if (sort_album)
 			dmap_structure_add (mlit, DMAP_CC_ASSU, sort_album);
 		else
 			g_warning ("Sort album requested but not available");
 	}
+	if (_dmap_share_client_requested (mb->bits, SONG_MEDIAKIND)) {
+		gint mediakind = 0;
+		g_object_get (record, "mediakind", &mediakind, NULL);
+		dmap_structure_add (mlit, DMAP_CC_AEMK, mediakind);
+	}
 
-	return;
+	return ((DMAPStructureItem *) mlit->data)->size;
 }
 
 static void
 genre_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
 {
 	const gchar *genre;
-	g_object_get (record, "daap.songgenre", &genre, NULL);
+	g_object_get (record, "songgenre", &genre, NULL);
+	if (!genre)
+		return;
 	if (! g_hash_table_lookup (ht, genre))
 		g_hash_table_insert (ht, (gchar *) genre, NULL);
 }
@@ -697,7 +732,9 @@ static void
 artist_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
 {
 	const gchar *artist;
-	g_object_get (record, "daap.songartist", &artist, NULL);
+	g_object_get (record, "songartist", &artist, NULL);
+	if (!artist)
+		return;
 	if (! g_hash_table_lookup (ht, artist))
 		g_hash_table_insert (ht, (gchar *) artist, NULL);
 }
@@ -706,13 +743,15 @@ static void
 album_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
 {
 	const gchar *album;
-	g_object_get (record, "daap.songalbum", &album, NULL);
+	g_object_get (record, "songalbum", &album, NULL);
+	if (!album)
+		return;
 	if (! g_hash_table_lookup (ht, album))
 		g_hash_table_insert (ht, (gchar *) album, NULL);
 }
 
 static void
-add_to_category_listing (gpointer key, gpointer value, gpointer user_data)
+add_to_category_listing (gpointer key, gpointer user_data)
 {
 	GNode *mlit;
 	GNode *node = (GNode *) user_data;
@@ -748,6 +787,7 @@ databases_browse_xxx (DMAPShare *share,
 	const gchar *browse_category;
 	GHashTable *category_items;
 	DMAPContentCode category_cc;
+	GList *values;
 
 	rest_of_path = strchr (path + 1, '/');
 	browse_category = rest_of_path + 10;
@@ -783,9 +823,15 @@ databases_browse_xxx (DMAPShare *share,
 
 	node = dmap_structure_add (abro, category_cc);
 
-	g_hash_table_foreach (category_items,
-			      add_to_category_listing,
-			      node);
+	values = g_hash_table_get_keys (category_items);
+	if (values && g_hash_table_lookup (query, "include-sort-headers")) {
+		g_debug ("Sorting...");
+		values = g_list_sort (values, (GCompareFunc) g_ascii_strcasecmp);
+	}
+
+	g_list_foreach (values, add_to_category_listing, node);
+	
+	g_list_free (values);
 
 	_dmap_share_message_set_from_dmap_structure (share, msg, abro);
 	dmap_structure_destroy (abro);
@@ -808,7 +854,7 @@ databases_items_xxx (DMAPShare *share,
 	const gchar *transcode_mimetype;
 	const gchar *rest_of_path;
 	const gchar *id_str;
-	gint id;
+	guint id;
 	const gchar *range_header;
 	guint64 filesize;
 	guint64 offset = 0;
@@ -816,7 +862,7 @@ databases_items_xxx (DMAPShare *share,
 
 	rest_of_path = strchr (path + 1, '/');
 	id_str = rest_of_path + 9;
-	id = atoi (id_str);
+	id = strtoul (id_str, NULL, 10);
 
 	g_object_get (share, "db", &db, NULL);
 	record = DAAP_RECORD (dmap_db_lookup_by_id (db, id));
